@@ -60,9 +60,11 @@ const MainWindow = new Lang.Class({
         this._configureId = 0;
         let ui = Utils.getUIObject('main-window', [ 'app-window',
                                                     'window-content',
-                                                    'search-entry' ]);
+                                                    'search-entry',
+                                                    'search-completion']);
         let grid = ui.windowContent;
         this._searchEntry = ui.searchEntry;
+        this._searchCompletion = ui.searchCompletion;
         this.window = ui.appWindow;
         this.window.application = app;
 
@@ -112,27 +114,29 @@ const MainWindow = new Lang.Class({
         this._searchEntry.connect('changed',
                                   this._searchPopup.hide.bind(this._searchPopup));
 
-        // FIXME: do as much of the following from UI file as possible
-        let completion = new Gtk.EntryCompletion({ model: this._placeStore,
-                                                   popup_completion: true,
-                                                   inline_completion: false,
-                                                   minimum_key_length: 2 });
-
-        let renderer = new Gtk.CellRendererPixbuf({ xpad: 2 });
-        completion.pack_start(renderer, false);
-        completion.add_attribute(renderer, "pixbuf", PlaceStore.Columns.PLACE_ICON);
-        completion.reorder(renderer, 0);
-        completion.set_text_column(PlaceStore.Columns.NAME);
-        let textCell = completion.get_cells()[0];
-        textCell.xpad = textCell.ypad = 4;
-
-        completion.connect('match-selected', (function(completion, model, iter) {
-            let place = model.get_value(iter, PlaceStore.Columns.PLACE);
-
-            this.mapView.showNGotoLocation(place.location);
+        this._searchCompletion.set_model(this._placeStore);
+        this._searchCompletion.connect('match-selected', (function(c, m, iter) {
+            let place = m.get_value(iter, PlaceStore.Columns.PLACE);
+            this.mapView.showNGotoLocation(place);
+            this._placeStore.addRecent(place);
         }).bind(this));
 
-        this._searchEntry.set_completion(completion);
+        this._searchCompletion.set_match_func(function(completion, key, iter) {
+            let model = completion.get_model();
+            let name = model.get_value(iter, PlaceStore.Columns.NAME);
+
+            if (name === null)
+                return false;
+
+            name = GLib.utf8_normalize (name, -1, GLib.NormalizeMode.ALL);
+            if (name === null)
+                return false;
+
+            if (!GLib.ascii_strncasecmp(name, key, key.length))
+                return true;
+            else
+                return false;
+        });
     },
 
     _initActions: function() {
@@ -218,6 +222,7 @@ const MainWindow = new Lang.Class({
 
         this._configureId = Mainloop.timeout_add(_CONFIGURE_ID_TIMEOUT, (function() {
             this._saveWindowGeometry();
+            this._configureId = 0;
             return false;
         }).bind(this));
     },
@@ -253,7 +258,7 @@ const MainWindow = new Lang.Class({
         let model = this._searchPopup.getModel();
         let place = model.get_value(iter, SearchResults.COL_PLACE);
 
-        this.mapView.showNGotoLocation(place.location);
+        this.mapView.showNGotoLocation(place);
 
         this._placeStore.addRecent(place);
         this._searchPopup.hide();
