@@ -53,7 +53,6 @@ const PlaceStore = new Lang.Class({
     Extends: Gtk.ListStore,
 
     _init: function() {
-        this._dirty = false;
         this.recentLimit = Application.settings.get('recent-places-limit');
         this._numRecent = 0;
         this.filename = GLib.build_filenamev([GLib.get_user_data_dir(),
@@ -66,6 +65,8 @@ const PlaceStore = new Lang.Class({
                                GObject.TYPE_STRING,
                                GObject.TYPE_INT,
                                GObject.TYPE_DOUBLE]);
+
+        this.set_sort_column_id(Columns.ADDED, Gtk.SortType.ASCENDING);
     },
 
     addFavorite: function(place) {
@@ -82,16 +83,25 @@ const PlaceStore = new Lang.Class({
     },
 
     addRecent: function(place) {
-        if (this._exists(place, PlaceType.RECENT))
+        if (this._exists(place, PlaceType.RECENT)) {
+            this._updateAddTime(place);
             return;
+        }
 
         if (this._numRecent === this.recentLimit) {
-            // Since all we do is append, the oldest recent will be
+            // Since we sort by added, the oldest recent will be
             // the first one we encounter.
             this._removeIf((function(model, iter) {
                 let type = model.get_value(iter, Columns.TYPE);
-                return type === PlaceType.RECENT;
-            }), true);
+
+                if (type === PlaceType.RECENT) {
+                    let name = model.get_value(iter, Columns.NAME);
+                    this._typeTable[name] = null;
+                    this._numRecent--;
+                    return true;
+                }
+                return false;
+            }).bind(this), true);
         }
         this._addPlace(place, PlaceType.RECENT, new Date().getTime());
         this._numRecent++;
@@ -118,7 +128,14 @@ const PlaceStore = new Lang.Class({
                 let place = Geocode.Place.new_with_location(obj.name,
                                                             obj.place_type,
                                                             location);
-
+                if (obj.bounding_box) {
+                    place.set_bounding_box(new Geocode.BoundingBox({
+                        top: obj.bounding_box.top,
+                        bottom: obj.bounding_box.bottom,
+                        left: obj.bounding_box.left,
+                        right: obj.bounding_box.right
+                    }));
+                }
                 this._addPlace(place, obj.type, obj.added);
                 if (obj.type === PlaceType.RECENT)
                     this._numRecent++;
@@ -128,10 +145,7 @@ const PlaceStore = new Lang.Class({
         }
     },
 
-    store: function() {
-        if (!this._dirty)
-            return;
-
+    _store: function() {
         let jsonArray = [];
         this.foreach(function(model, path, iter) {
             let place    = model.get_value(iter, Columns.PLACE),
@@ -139,23 +153,32 @@ const PlaceStore = new Lang.Class({
                 type     = model.get_value(iter, Columns.TYPE),
                 added    = model.get_value(iter, Columns.ADDED);
 
+            let bounding_box = null;
+            if (place.bounding_box !== null) {
+                bounding_box = {
+                    top: place.bounding_box.top,
+                    bottom: place.bounding_box.bottom,
+                    left: place.bounding_box.left,
+                    right: place.bounding_box.right
+                };
+            }
+
             jsonArray.push({
-                place_type: place.place_type,
-                name:       place.name,
-                latitude:   location.latitude,
-                longitude:  location.longitude,
-                altitude:   location.altitude,
-                accuracy:   location.accuracy,
-                type:       type,
-                added:      added
+                place_type:   place.place_type,
+                name:         place.name,
+                latitude:     location.latitude,
+                longitude:    location.longitude,
+                altitude:     location.altitude,
+                accuracy:     location.accuracy,
+                bounding_box: bounding_box,
+                type:         type,
+                added:        added
             });
         });
 
         let buffer = JSON.stringify(jsonArray);
         if (!Utils.writeFile(this.filename, buffer))
-            throw new Error('failed to write file');
-        else
-            this.dirty = false;
+            log('Failed to write places file!');
     },
 
     _addPlace: function(place, type, added) {
@@ -177,13 +200,7 @@ const PlaceStore = new Lang.Class({
             }).bind(this));
         }
         this._typeTable[place.name] = type;
-        this._dirty = true;
-
-        try {
-            this.store();
-        } catch (e) {
-            Utils.debug(e);
-        }
+        this._store();
     },
 
     _exists: function(place, type) {
@@ -199,5 +216,18 @@ const PlaceStore = new Lang.Class({
             }
             return false;
         }).bind(this));
+    },
+
+    _updateAddTime: function(place) {
+        this.foreach((function(model, path, iter) {
+            let name = model.get_value(iter, Columns.NAME);
+
+            if (name === place.name) {
+                let updated = new Date().getTime();
+                model.set_value(iter, Columns.ADDED, updated);
+                this._store();
+                return;
+            }
+        }).bind(this));
     }
 });
